<?php

namespace Drupal\tmgmt_contentapi\Util;

use \TMGMTJob;
use  Exception;
use stdClass;

/**
 * CreateToken Class Doc Comment.
 *
 * @category Class
 * @package Drupal\tmgmt_contentapi\Util
 * @author Arben Sabani
 */
class GeneralHelper {

  /**
   * Get label of the job.
   *
   * @param \TMGMTJob $job
   *   Job.
   *
   * @return string
   *   stingyfied label.
   */
  public static function getJobLabel(\TMGMTJob $job) {
    return $job->label;
  }

  /**
   * Create file object from uri.
   *
   * @param string $uri
   *   Uri.
   *
   * @return \stdClass
   *   file class object.
   */
  public static function createFileObject($uri) {
    global $user;
    $file = new stdClass();
    $file->uid = $user->uid;
    $file->filename = drupal_basename($uri);
    $file->uri = $uri;
    $file->filemime = file_get_mimetype($uri);
    $file->filesize = filesize($uri);
    $file->status = 1;
    return $file;//file_save($file);
  }

  /**
   * Remove spec. characters from label.
   *
   * @param \TMGMTJob $job
   *   Job.
   *
   * @return string
   *   stingyfied label.
   */
  public static function getJobLabelNoSpeChars(\TMGMTJob $job) {
    $toreturn = $job->label;
    return GeneralHelper::getStringNoSpeChars($toreturn);
  }

  /**
   * Remove spec. characters from label.
   *
   * @param string $arg
   *   Job.
   *
   * @return string
   *   removed spec. chars string.
   */
  public static function getStringNoSpeChars($arg) {
    $toreturn = mb_ereg_replace("([^\w\s\d\-_~,;\[\]\(\).])", '', $arg);
    // Remove any runs of periods (thanks falstro!)
    $toreturn = mb_ereg_replace("([\.]{2,})", '', $toreturn);
    $toreturn = filter_xss($toreturn);
    return $toreturn;
  }


  /**
   * Create file object from uri.
   *
   *
   */
  public static function addCpaSettingsToJob(\TMGMTJob $job, $cpsettings) {
    $job->settings['"capi-remote"'] = $cpsettings;
  }

  /**
   * Get all jobs for connector config.
   *
   * @param \TMGMTTranslator $translator
   *   TranslatorInterface.
   *
   * @return mixed
   *   A storage instance.
   */
  public static function getAllJobsByTranslator(\TMGMTTranslator $translator){
    $trname = $translator->name;
    $trjobs = entity_load('tmgmt_job', FALSE, ['translator' => $trname]);
    return $trjobs;
  }

  /**
   * Get all jobs for connector config.
   *
   * @param \TMGMTJob $job
   *   TranslatorInterface.
   *
   * @return string
   *   Id of contentapi Job.
   */
  public static function getCpJobIdfromLocJob(\TMGMTJob $job){
    $jobcpsettings = unserialize($job->getSetting('"capi-remote"'));
    $firstrequst = array_values($jobcpsettings)[0][0];
    if (!isset($firstrequst)){
      throw new Exception("Job Id could not be found in local job!");
    }
    else{
      return $firstrequst->getJobId();
    }
  }

  /**
   * Reset Job and Items to state Active.
   *
   * @param \TMGMTJob $job
   *   TranslatorInterface.
   * @param \stdClass $file
   *   TranslatorInterface.
   */
  public static function resetJobandItemsToActive(\TMGMTJob &$job, \stdClass $file){
    $itemsToset = $job->getItems();
    $loadedxml = simplexml_load_file(drupal_realpath($file->uri));
    $loadedxml->registerXPathNamespace('xliff', 'urn:oasis:names:tc:xliff:document:1.2');
    foreach ($itemsToset as $item) {
      // If the xlf caontains one of job's items, set state to active. [@phase-name='extraction']
      $tjiid = $item->tjiid;
      $groups = $loadedxml->xpath("//xliff:group[@id='" . $tjiid ."']");
      if(count($groups) == 1){
        $item->setState(TMGMT_JOB_ITEM_STATE_ACTIVE);
        $job->setState(TMGMT_JOB_STATE_ACTIVE);
      }
    }
  }

  public static function checkFileUsageForFile(\stdClass $file){
    $query = $result = db_select('file_managed', 'f')
      ->fields('f', array('fid', 'filename', 'uri', 'filemime', 'uid', 'filesize', 'status'))
      ->condition('uri', $file->uri);
    $result = $query -> execute();
    $toreturn = [];
    foreach ($result as $usage) {
      array_push($toreturn,$usage);
    }
    return $toreturn;
  }

}
