<?php

/**
 * @file
 * Provides the file translator plugin controller.
 */

use Drupal\tmgmt_contentapi\Swagger\Client\Model\ProviderId;
use Drupal\tmgmt_contentapi\Swagger\Client\Model\CreateRequestFile;
use Drupal\tmgmt_contentapi\Swagger\Client\Model\CreateToken;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\TokenApi;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\JobApi;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\RequestApi;

use Drupal\tmgmt_contentapi\Swagger\Client\Api\SourceFileApi;
use Drupal\tmgmt_contentapi\Util\ConentApiHelper;
use Drupal\tmgmt_contentapi\Util\GeneralHelper;
/**
 * File translator plugin controller.
 */
class TmgmtContentapiTranslatorPluginController extends TMGMTDefaultTranslatorPluginController {

  /**
   * {@inheritdoc}
   */
  public function canTranslate(TMGMTTranslator $translator, TMGMTJob $job) {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function requestTranslation(TMGMTJob $job) {
    // Defined here as they will be used in catch block.
    $createdcpjob = NULL;
    $token = NULL;
    $zipPath = NULL;
    $ziparchive = NULL;
    $jobapi = NULL;
    $allfilespath = NULL;
    try {
      // Message which will be displayed using drupa_set_message to disaplay download.
      $messageTopass = 'Exported files can be downloaded here: <br/>';
      $translator = $job->getTranslator();
      $contentapisettings = $translator->getSetting("capi-settings");
      $oneexportfile = $translator->getSetting("one_export_file");
      $token = $contentapisettings['connection_message']['token'];
      if ($job->getSetting('capi-settings')["task"] == "trans") {
        // Export files: gernerate paths and other variables.
        $exporter = new TmgmtContentapiFileFormatXLIFF();
        $joblabel = $job->label;
        $sourc_lang = $job->source_language;
        $target_lang = $job->target_language;
        $remotemappings = $this->getRemoteLanguagesMappings($job->getTranslator());
        $cpsource_lang = $remotemappings[$sourc_lang];
        $cptarget_lang = $remotemappings[$target_lang];
        $dirnameallfiles = $joblabel . '_' . $job->tjid . "_" . $cpsource_lang. "_" . $cptarget_lang;
        $zipName = 'zip_job_' . $dirnameallfiles . '.zip';
        $allfilespath = $job->getSetting('scheme') . '://tmgmt_contentapi/LioxSentFiles/' . $dirnameallfiles;
        $zipPath = $allfilespath . "/" . $zipName;
        $filearraytodelte = array();
        $filearrayexportedfiles = array();
        $transferfiles = array();
        // Create folder where all exported files will be stored.
        if (file_prepare_directory($allfilespath, FILE_CREATE_DIRECTORY)) {
          // Export each item of the job in same file.
          if($oneexportfile){
            $labelname = GeneralHelper::getStringNoSpeChars($joblabel);
            $name = $labelname . "_" . $job->tjid . "_all_"  . $cpsource_lang . '_' . $cptarget_lang . '.xlf';
            $jobpath = $allfilespath . "/" . $name;
            $file = file_save_data($exporter->export($job), $jobpath, FILE_EXISTS_REPLACE);
            $filearraytodelte[] = $file;
            $filearrayexportedfiles['all'] = $file;
          }
          else {
            // Export each item of the job in separate file.
            foreach ($job->getItems() as $item) {
              $labelname = GeneralHelper::getStringNoSpeChars($item->label());
              $name = $labelname . "_" . $job->tjid . "_" . $item->tjiid . "_" . $cpsource_lang . '_' . $cptarget_lang . '.xlf';
              $itempath = $allfilespath . "/" . $name;
              $file = file_save_data($exporter->exportItem($item), $itempath, FILE_EXISTS_REPLACE);
              $filearraytodelte[] = $file;
              $filearrayexportedfiles[$item->tjiid] = $file;
            }
          }
          // Zip the exported files.
          $ziparchive = new ZipArchive();
          $openresult = $ziparchive->open(drupal_realpath($zipPath), ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE);
          $zipcloseresult = FALSE;
          if ($openresult) {
            foreach ($filearrayexportedfiles as $tempfile) {
              $ziparchive->addFile(drupal_realpath($tempfile->uri), $tempfile->filename);
            }
            $zipcloseresult = $ziparchive->close();
            if ($zipcloseresult) {
              $zipfileobj = $this->createFileObjectFromZip($zipPath);
              //file_usage_add($zipfileobj, 'tmgmt_contentapi', 'tmgmt_job', $job->tjid);
            }
          }
          else {

          }
          // If exported files are transfered as zip, delete org. exports as already in the zip.
          if ($job->getTranslator()->getSetting('transfer-settings')) {
            // Add zip to transfer array.
            $transferfiles['all'] = $this->createFileObjectFromZip($zipPath);
            foreach ($filearrayexportedfiles as $tempfile) {
              file_unmanaged_delete($tempfile->uri);
            }
          }
          else {
            // All exported files added to transfer array.
            $transferfiles = $filearrayexportedfiles;
          }
          // Create Job in Content API.
          $contentapibundle = array();
          $jobrequest = ConentApiHelper::genrateJobRequst($job);
          $jobapi = new JobApi();
          $createdcpjob = NULL;
          try {
            $createdcpjob = $jobapi->jobsPost($token, $jobrequest);
          }
          catch (Exception $e) {
            $job->addMessage($e->getMessage(), array(), 'error');
            return;
          }
          // Upload files from transferfiles array to CPA.
          $fileapi = new SourceFileApi();
          $transrequestapi = new RequestApi();
          // Array to store temporally translation requst and associated files.
          $contentapitrrequstfiles = array();
          foreach ($transferfiles as $id => $tmpfile) {
            $data = array();
            $data["job_id"] = $createdcpjob->getJobId();
            $data["filename"] = $tmpfile->filename;
            $data["filetype"] = $tmpfile->filemime;
            //$stmrg = \Drupal::service('stream_wrapper_manager')->getViaUri($tmpfile->getFileUri());
            $extpath = drupal_realpath($tmpfile->uri);
            $filrequst = new SplFileObject($extpath);
            $contentapitmpfile = $fileapi->jobsJobIdUploadPost($token, $data["job_id"], $data["filename"], $data["filetype"], $filrequst);
            $request = new CreateRequestFile(array(
              "request_name" => $job->tjid . "_" . $id . "_" .  $cpsource_lang . "-" . $cptarget_lang,
              "source_native_id" => $job->tjid . "_" . $id,
              "source_native_language_code" => $cpsource_lang,
              "target_native_language_codes" => array($cptarget_lang),
              "file_id" => $contentapitmpfile->getFileId(),
            ));
            $contentapitmprequst = $transrequestapi->jobsJobIdRequestsAddfilePost($token, $data["job_id"], $request);
            $contentapibundle[] = $contentapitmprequst;
            $filrequst = NULL;
          }
          // Submit content api job.
          $pr = $job->getSetting("capi-settings")["provider"];
          $jid = $createdcpjob->getJobId();
          $prmodel = new ProviderId(array('provider_id' => $pr));
          $jobsubmitresult = $jobapi->jobsJobIdSubmitPut($token, $jid, $prmodel);
          if ($jobsubmitresult) {
            // TODO
            // Add content api bundle to job's settings.
            GeneralHelper::addCpaSettingsToJob($job, serialize($contentapibundle));
            $messageTopass .= '<a href="' . file_create_url($zipPath) . '">' . filter_xss(GeneralHelper::getJobLabelNoSpeChars($job)) . '</a>';
            drupal_set_message($messageTopass);
            if($job->getSetting('capi-settings')['quote']['is_quote']){
              $job->submitted("This job was submitted for a quote. To submit your job for processing, you must log into your translation provider's system to approve this quote.");
            }
            else{
              $job->submitted("Job sent to provider!");
            }

            // Delete files after sending to Provider.
            foreach ($filearraytodelte as $tempfile) {
              file_unmanaged_delete($tempfile->uri);
            }
          }
        }
        else {
          throw new Exception("Could not create directory for export: " . $allfilespath);
        }
      }
      else {
        // TM update is not supported yet, when creating new job!!
        drupal_set_message(t('TM update not supported!'),'warning');
        return;
      }
    }
    catch (Exception $exception) {
      // If exception occurs, clean up everything: delete exported files, cancel job in CA if any.
      foreach ($filearraytodelte as $tempfile) {
        if (file_exists($tempfile->uri)) {
          file_unmanaged_delete($tempfile->uri);
        }
      }
      if (!file_exists($zipPath) && $ziparchive != NULL) {
        $ziparchive->close();
      }
      $job->setState(TMGMT_JOB_STATE_UNPROCESSED);
      try {
        if (isset($createdcpjob)) {
          // TODO: check why result is NULL, only at certain state job can be deleted? But seems to work!!
          $deletresult = $jobapi->jobsJobIdDelete($token, $createdcpjob->getJobId());
        }
      }
      catch (Exception $ex) {
        drupal_set_message("Tried to delete Job in Content API, but failed: " . $ex->getMessage(), "error");
      }
      // TODO: Check why zip cannot be deleted. But after two tests seems to work?
      $zipfileobj = $this->createFileObjectFromZip($zipPath);
      file_unmanaged_delete($zipfileobj->uri);
      file_unmanaged_delete_recursive($allfilespath);
      drupal_set_message($exception->getMessage(), "error");
    }
  }

  /**
   * Comment.
   *
   * @param string $uri
   *   Comment.
   *
   * @return \stdClass
   *   Comment.
   */
  public function createFileObjectFromZip($uri) {
    global $user;
    $file = new stdClass();
    $file->uid = $user->uid;
    $file->filename = drupal_basename($uri);
    $file->uri = $uri;
    $file->filemime = file_get_mimetype($uri);
    $file->filesize = filesize($uri);
    $file->status = 1;
    $testfiles = GeneralHelper::checkFileUsageForFile($file);
    if (count($testfiles) > 0){
     return array_pop($testfiles);
    }
    return file_save($file);
  }

  /**
   * Comment.
   *
   * @param array $fileArray
   *   Comment.
   * @param string $jobtjid
   *   Comment.
   */
  public function deletZippedFiles(array $fileArray, $jobtjid) {
    foreach ($fileArray as $file) {

      file_usage_delete($file, 'tmgmt_contentapi', 'tmgmt_job', $jobtjid);
      file_delete($file);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function abortTranslation(TMGMTJob $job) {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function hasCheckoutSettings(TMGMTJob $job) {
    return $job->getTranslator()->getSetting('allow_override');
  }

  /**
   * {@inheritdoc}
   */
  public function defaultSettings() {
    return array(
      'export_format' => 'contentapi_xlf',
      'allow_override' => TRUE,
      'scheme' => 'public',
      // Making this setting TRUE by default is more appropriate, however we
      // need to make it FALSE due to backwards compatibility.
      'xliff_processing' => FALSE,
      'xliff_cdata' => FALSE,
      'capi_username' => '',
      //'capi_password' => '',
      'one_export_file' => TRUE,
      'po_reference' => '',
      'transfer-settings' => FALSE
    );
  }

}
